#!/usr/bin/env python3
import glob
import dataclasses
import pathlib
import sys
import subprocess
import os
import logging

from typing import (Self, Optional,)

logger = logging.getLogger(__name__)

@dataclasses.dataclass
class BootstrapSettings:
    env_path: pathlib.Path
    python_path: pathlib.Path

    @classmethod
    def get(
        cls,
        base_dir: Optional[pathlib.Path] = None,
    ) -> Self:
        if base_dir is None:
            base_dir = pathlib.Path.cwd()

        env_path = base_dir / '.venv'
        python_path = env_path / 'bin' / 'python3'

        return cls(
            env_path=env_path,
            python_path=python_path,
        )

def env_bootstrap() -> None:
    bootstrap_settings = BootstrapSettings.get()

    subprocess.check_call([
        'uv', 'venv', '--seed', '--offline',
        str(bootstrap_settings.env_path)
    ])

    subprocess.check_call([
        'uv',
        'pip',
        'install',
        '-p',
        bootstrap_settings.python_path,
        '--offline',
        'uv',
    ])

    subprocess.check_call([
        bootstrap_settings.python_path,
        '-m',
        'uv', 'pip', 'install',
        '--offline',
        'build', 'setuptools', 'meson-python', 'pybind11',
    ])

    early_wheels = glob.glob(
        str(
            pathlib.Path(__file__).parent / 'deps' / 'dist' / 'early' / '*.whl'
        )
    )
    if len(early_wheels) > 0:
        subprocess.check_call([
            bootstrap_settings.python_path,
            '-m',
            'uv', 'pip', 'install',
            '--offline',
            *early_wheels,
        ])

def paths_equal(
    a: pathlib.Path | str,
    b: pathlib.Path | str
) -> bool:
    return (
        os.path.abspath(str(a))  ==
        os.path.abspath(str(b))
    )

def run() -> None:
    bootstrap_settings = BootstrapSettings.get()

    logging.basicConfig(level=logging.INFO)

    if not bootstrap_settings.env_path.exists():
        env_bootstrap()

    logger.info([sys.executable, sys.argv, bootstrap_settings.python_path])

    if not paths_equal(sys.executable, bootstrap_settings.python_path):
        os.execv(
            str(bootstrap_settings.python_path),
            [
                str(bootstrap_settings.python_path),
                *sys.argv,
            ]
        )

    os.execv(
        str(bootstrap_settings.python_path),
        [
            str(bootstrap_settings.python_path),
            pathlib.Path(__file__).parent / 'cli.py',
            *sys.argv[1:],
        ]
    )


if __name__ == '__main__':
    run()